//--------------------------------------------------------------------------------
// Copyright (c) 2001-2021 by PDFTron Systems Inc. All Rights Reserved.
//--------------------------------------------------------------------------------
using System;
using System.Runtime.InteropServices;
using System.Collections;

namespace test
{

	class Class1
	{
        public delegate Int32 PDFSecureCallback(int mode, string msg, IntPtr user_data);

        // The following three lines import the PDFSecure functions from pdfsecure.dll (via PInvoke).
		[DllImport("pdfsecure.dll")]
		public static extern int PDFSecureInit(string user_name, string company, string license_key);

		[DllImport("pdfsecure.dll")]
		public static extern int PDFSecureRun(string command_str, PDFSecureCallback funct, IntPtr user_data);

		// You can modify the following lines with your registration information.
		static string username = "John Doe";
		static string company = "My Company";
		static string lic_key = "my license key";

		static int PDFSECURE_ERROR = 1;    // Show the error message
		static int PDFSECURE_MSG = 2;      // Report the message
		static int PDFSECURE_GETPASS = 3;  // Get the password
		static int PDFSECURE_OUT_FILENAME=4; // Get the output 

		// A custom callback that can be used to report errors and other messages.
		public static Int32 MyCallback(int mode, string msg, IntPtr user_data)
		{
			if (mode == PDFSECURE_ERROR) {
				Console.WriteLine("Error: {0}", msg);
			}
			else if (mode == PDFSECURE_MSG) {
				Console.Write("{0}", msg);
			}
			else if (mode == PDFSECURE_GETPASS) {
				//gl_pass = Console.ReadLine();
				// return gl_pass.c_str();
			}
			else if(mode == PDFSECURE_OUT_FILENAME)
			{
				//check whether the IntPtr is valid
				if(user_data.ToInt32() != 0)
				{
					//cast it to a GCHandle to
					//obtain the ArrayList
					GCHandle gch = (GCHandle)user_data;
					ArrayList list = (ArrayList)gch.Target;
					//add the newly created file's path
					list.Add(msg);
				}
			}

			return 0;
		}

		static void Main(string[] args)
		{
			PDFSecureInit(username, company, lic_key);
			PDFSecureCallback mycallback = new PDFSecureCallback(MyCallback);

			 // Variable used to switch between different code samples.
			int example = 1;

			if (example == 1) 
			{
				/// The simplest PDFSecure program: Pass an explicit command-line string.
				/// 
				/// This example will secure all PDF documents in 'TestFiles' using 
				/// 128-bit encryption. The password required to open processed files is 
				/// 'secret'. 
				/// 
				/// - The -o parameter is used to specify the output folder/file. 
				/// - The -s parameter specifies that the output document should be encrypted 
				/// using 128-bit RC4 encryption.
				/// - The -u parameter specifies the password required to access the secured 
				/// document.
				/// 
				/// For a detailed explanation of all commands, please refer to PDFSecure 
				/// User Manual.
				PDFSecureRun("-o TestOutput/test1 -s 128 -u secret TestFiles", mycallback, new IntPtr(0));
			}
			else if (example == 2) 
			{
				// An example of how to use PDFSecure SDK to implement a command-line 
				// application for processing of PDF documents.
				String s = "";
				foreach (string arg in args)
				{
					s += arg + " ";
				}

				PDFSecureRun(s, mycallback, new IntPtr(0));
			}
			else if (example == 3) 
			{
				// A more comlicated example that illustrates how to dynamically build a command 
				// string based on user security settings...
				// Please note that this sample does not cover all possible options offered by 
				// PDFSecure. For a detailed explanation of all commands, please refer to PDFSecure 
				// User Manual.
				bool linearized_output = true;
				bool aes_encryption = false;
				string output_folder = "TestOutput/test1";
				string open_password = "mypass";
				string title = "My Title";
				string author = "Joe Doe";
				string subject = "My Subject";
				string keywords = "key1 key2 key2";
				string creator = "PDFTron PDFNet";
				string producer = "PDFSecure";

				// Permissions
				bool printing = false;
				bool high_rez_printing = false;
				bool doc_modifications = true;  // Changing the document
				bool content_extraction = true;
				bool commenting = true;
				bool forms_editing = true;
				bool accessibility = true;
				bool document_assembly = true;

				// Given the above settings build a command string.
				String s = "";

				if (output_folder != "") 
					s += "-o " + output_folder + " ";

				if (aes_encryption) 
				{
					s += "-s AES ";  // Use 128-bit AES (Advanced Encryption Standard); Supported in PDF 1.6 (Acrobat 7) and above.
				}
				else 
				{
					s += "-s 128 ";  // Use 128-bit RC4 encryption. Supported in PDF 1.4 (Acrobat 5) and above.
					// To remove security use:
					//  s += "-s R";  etc.
				}

				if (open_password != "") 
					s += "--userpass " + open_password + " ";

				// Set the permissions.
				s += "-e a -d \"";
				if (printing == false) s += "p";
				if (high_rez_printing == false) s += "h";
				if (doc_modifications == false) s += "m";
				if (content_extraction == false)  s += "c";
				if (commenting == false)  s += "o";
				if (forms_editing == false)  s += "f";
				if (accessibility == false)  s += "x";
				if (document_assembly == false)  s += "s";
				s += "\" ";

				if (linearized_output) s += "-l ";

				// Set document information.
				if (title != "")    s += "--title \"" + title + "\" ";
				if (author != "")   s += "--author \"" + author + "\" ";
				if (subject != "")  s += "--subject \"" + subject + "\" ";
				if (keywords != "") s += "--keywords \"" + keywords + "\" ";
				if (creator != "")  s += "--creator \"" + creator + "\" ";
				if (producer != "") s += "--producer \"" + producer + "\" ";

				// specify input files and folders...
				s += "TestFiles/black.pdf ";
				s += "TestFiles/blue.pdf "; 
				// ...


				//an arraylist to store the saved file path(s)
				ArrayList saved_files=new ArrayList();

				//create a GCHandle to allow MyCallback to access
				//the object saved_files
				GCHandle gch = GCHandle.Alloc(saved_files);
				
				// Execute the command string.
				PDFSecureRun(s, mycallback, (IntPtr)gch);
				gch.Free();

				//print the saved file(s)
				Console.WriteLine("The Saved Files Are:");
				for(int i=0; i<saved_files.Count; i++)
				{
					Console.WriteLine(saved_files[i]);
				}
			}

			Console.WriteLine("Done.");
		}
	}
}
